package org.springframework.boot.loader.jar;

package .var.folders._j.vfbv187d0bb8b8w5spzgg8180000gn.T;

import java.io.ByteArrayOutputStream;
import java.io.UnsupportedEncodingException;
import java.net.URLConnection;
import java.net.URLEncoder;
import org.springframework.boot.loader.jar.AsciiBytes;
import org.springframework.boot.loader.jar.JarURLConnection;
import org.springframework.boot.loader.jar.StringSequence;

class JarEntryName {
  private final StringSequence name;
  
  private String contentType;
  
  JarEntryName(StringSequence spec) {
    this.name = decode(spec);
  }
  
  private StringSequence decode(StringSequence source) {
    if (source.isEmpty() || source.indexOf('%') < 0)
      return source; 
    ByteArrayOutputStream bos = new ByteArrayOutputStream(source.length());
    write(source.toString(), bos);
    return new StringSequence(AsciiBytes.toString(bos.toByteArray()));
  }
  
  private void write(String source, ByteArrayOutputStream outputStream) {
    int length = source.length();
    for (int i = 0; i < length; i++) {
      int c = source.charAt(i);
      if (c > 127) {
        try {
          String encoded = URLEncoder.encode(String.valueOf((char)c), "UTF-8");
          write(encoded, outputStream);
        } catch (UnsupportedEncodingException ex) {
          throw new IllegalStateException(ex);
        } 
      } else {
        if (c == 37) {
          if (i + 2 >= length)
            throw new IllegalArgumentException("Invalid encoded sequence \"" + source
                .substring(i) + "\""); 
          c = decodeEscapeSequence(source, i);
          i += 2;
        } 
        outputStream.write(c);
      } 
    } 
  }
  
  private char decodeEscapeSequence(String source, int i) {
    int hi = Character.digit(source.charAt(i + 1), 16);
    int lo = Character.digit(source.charAt(i + 2), 16);
    if (hi == -1 || lo == -1)
      throw new IllegalArgumentException("Invalid encoded sequence \"" + source.substring(i) + "\""); 
    return (char)((hi << 4) + lo);
  }
  
  CharSequence toCharSequence() {
    return (CharSequence)this.name;
  }
  
  public String toString() {
    return this.name.toString();
  }
  
  boolean isEmpty() {
    return this.name.isEmpty();
  }
  
  String getContentType() {
    if (this.contentType == null)
      this.contentType = deduceContentType(); 
    return this.contentType;
  }
  
  private String deduceContentType() {
    String type = isEmpty() ? "x-java/jar" : null;
    type = (type != null) ? type : URLConnection.guessContentTypeFromName(toString());
    type = (type != null) ? type : "content/unknown";
    return type;
  }
  
  static JarEntryName get(StringSequence spec) {
    return get(spec, 0);
  }
  
  static JarEntryName get(StringSequence spec, int beginIndex) {
    if (spec.length() <= beginIndex)
      return JarURLConnection.access$000(); 
    return new JarEntryName(spec.subSequence(beginIndex));
  }
}
